@@ -1,10 +1,6 @@
 from terra_sdk.core.distribution.proposals import CommunityPoolSpendProposal
 from terra_sdk.core.gov.proposals import TextProposal
 from terra_sdk.core.params.proposals import ParameterChangeProposal
-from terra_sdk.core.treasury.proposals import (
-    RewardWeightUpdateProposal,
-    TaxRateUpdateProposal,
-)
 
 from .base import create_demux
 
@@ -13,7 +9,5 @@ parse_content = create_demux(
         CommunityPoolSpendProposal,
         TextProposal,
         ParameterChangeProposal,
-        RewardWeightUpdateProposal,
-        TaxRateUpdateProposal,
     ]
 )
